/* Konfiguration fr das Rx-Backend-Board (Freescale Kinetis MK02FN128VLH10) */

#include "StdTypes.h"
#include "MK02FN.h"
#include "startup.h"
#include "config.h"

/* Port-Pin Eigenschaften */
#define NOPIN    0
#define DISABLED 0

#define CLEARINTSTATE (1<<24)

#define DMA_LtoH (1<<16)
#define DMA_HtoL (2<<16)
#define DMA_BOTH (3<<16)
#define INT_onLO (8<<16)
#define INT_onHI (12<<16)
#define INT_LtoH (9<<16)
#define INT_HtoL (10<<16)
#define INT_BOTH (11<<16)


#define LOCKME  (1<<15)
#define ALT0     0
#define ANALOG   0
#define ALT1   (1<<8)
#define ALT2   (2<<8)
#define ALT3   (3<<8)
#define ALT4   (4<<8)
#define ALT5   (5<<8)
#define ALT6   (6<<8)
#define ALT7   (7<<8)

#define STRONG    (1<<6)
#define OPENDRAIN (1<<5)
#define FILTER    (1<<4)
#define SLOW      (1<<2)

#define PULLUP 3
#define PULLDN 2



/****** Anschlsse auf dem Board, Ports A,B,C,D,E *******/
// merke: GPIO PDDR: 0 = IN, 1 = OUT


#define numPORTA  20
const dword wfPORTA_PCR[numPORTA] =
{ ALT7,                       // PA0   SWD_CLK
  DISABLED,                   // PA1   n.b.
  ALT1 | INT_BOTH,            // PA2   IN vom IR-Empfnger, nicht filtern!
  ALT7,                       // PA3   SWD_DIO
  DISABLED,                   // PA4   NMI
  ALT1 | FILTER,              // PA5   IN von TAS2
  NOPIN,                      //   6
  NOPIN,                      //   7
  NOPIN,                      //   8
  NOPIN,                      //   9
  NOPIN,                      //  10
  NOPIN,                      //  11
  ALT1 | FILTER,              // PA12  IN von TAS1
  ALT1 | FILTER,              // PA13  IN von TASD
  NOPIN,                      //   14
  NOPIN,                      //   15
  NOPIN,                      //   16
  NOPIN,                      //   17
  ALT0,                       // PA18  EXTAL0
  ALT0,                       // PA19  XTAL0
 };

#define wfGPIOA_PDDR    ((1<<19)  )  // nur XTAL0


#define numPORTB  20
const dword wfPORTB_PCR[numPORTB] =
{ ALT2,                       // PB0   SCL0
  ALT2,                       // PB1   SDA0
  ALT1 | FILTER,              // PB2   DrehGeber-B
  ALT1 | FILTER | INT_BOTH,   // PB3   DrehGeber-A
  NOPIN,                      //   4
  NOPIN,                      //   5
  NOPIN,                      //   6
  NOPIN,                      //   7
  NOPIN,                      //   8
  NOPIN,                      //   9
  NOPIN,                      //   10
  NOPIN,                      //   11
  NOPIN,                      //   12
  NOPIN,                      //   13
  NOPIN,                      //   14
  NOPIN,                      //   15
  ALT3,                       // PB16  RX0
  ALT3,                       // PB17  TX0
  ALT1,                       // PB18  LCD LOAD
  ALT1,                       // PB19  LCD DISP
};

#define wfGPIOB_PDDR    ((1<<17) | (1<<18) | (1<<19)  )


#define numPORTC  12
const dword wfPORTC_PCR[numPORTC] =
{ ALT1,                       // PC0   OUT LCD DATA 0
  ALT1,                       // PC1   OUT LCD DATA 1
  ALT1,                       // PC2   OUT LCD DATA 2
  ALT1,                       // PC3   OUT LCD DATA 3  auch CLKOUT
  ALT1,                       // PC4   OUT LCD DATA 4
  ALT1,                       // PC5   OUT LCD DATA 5
  ALT1,                       // PC6   OUT LCD DATA 6
  ALT1,                       // PC7   OUT LCD DATA 7
  ALT1,                       // PC8   OUT LCD  CP
  ALT1,                       // PC9   OUT LCD  M
  ALT1,                       // PC10  OUT LCD  YSCL
  ALT1,                       // PC11  OUT LCD  VSYNC
};

#define wfGPIOC_PDDR    (0xFFF  )


#define numPORTD  8
const dword wfPORTD_PCR[numPORTD] =
{ ALT2,                       // PD0   NRF SPI0CS
  ALT2,                       // PD1   NRF SPI0SCK
  ALT2,                       // PD2   NRF SPI0MOSI
  ALT2,                       // PD3   NRF SPI0MISO
  ALT1,                       // PD4   NRF IRQ
  ALT1,                       // PD5   NRF CE
  DISABLED,                   // PD6   an X4
  DISABLED,                   // PD7   an X4
};

#define wfGPIOD_PDDR    (1 | 2 | 4 | (1<<5) | (1<<4) )


#define numPORTE  26
const dword wfPORTE_PCR[numPORTE] =
{ ALT1,                       // PE0   an X4
  DISABLED,                   // PE1   an X4
  NOPIN,                      //   2
  NOPIN,                      //   3
  NOPIN,                      //   4
  NOPIN,                      //   5
  NOPIN,                      //   6
  NOPIN,                      //   7
  NOPIN,                      //   8
  NOPIN,                      //   9
  NOPIN,                      //   10
  NOPIN,                      //   11
  NOPIN,                      //   12
  NOPIN,                      //   13
  NOPIN,                      //   14
  NOPIN,                      //   15
  DISABLED,                   // PE16  an X4
  DISABLED,                   // PE17  an X4
  DISABLED,                   // PE18  an X4
  DISABLED,                   // PE19  an X4
  NOPIN,                      //   20
  NOPIN,                      //   21
  NOPIN,                      //   22
  NOPIN,                      //   23
  ALT1 | PULLUP,              // PE24  an X5
  ALT1 | PULLUP,              // PE25  an X5
};

#define wfGPIOE_PDDR    ( 1 | (1<<24) | (1<<25)  )



void sysconfig (void)
{ int i;

  // Pins aufsetzen
  SIM_SCGC5 |= 0x1F<<9;     // erstmal Clock an alle Ports legen

  i=0;  while (i<numPORTA) {PORTA_PCR[i] = wfPORTA_PCR[i]; ++i;}
  i=0;  while (i<numPORTB) {PORTB_PCR[i] = wfPORTB_PCR[i]; ++i;}
  i=0;  while (i<numPORTC) {PORTC_PCR[i] = wfPORTC_PCR[i]; ++i;}
  i=0;  while (i<numPORTD) {PORTD_PCR[i] = wfPORTD_PCR[i]; ++i;}
  i=0;  while (i<numPORTE) {PORTE_PCR[i] = wfPORTE_PCR[i]; ++i;}

  // Filter fr die Tasten + Drehgeber
  PORTA_DFCR = 1;      // Filterclock = 1 kHz
  PORTB_DFCR = 1;      // Filterclock = 1 kHz
  PORTA_DFWR = 20;     // 20 ms lang
  PORTB_DFWR = 20;     // 20 ms lang

  // Ports: Datenrichtungen
  GPIOA_PDDR = wfGPIOA_PDDR;
  GPIOB_PDDR = wfGPIOB_PDDR;
  GPIOC_PDDR = wfGPIOC_PDDR;
  GPIOD_PDDR = wfGPIOD_PDDR;
  GPIOE_PDDR = wfGPIOE_PDDR;


  // Quarz-Oszillator aufsetzen, derzeit 10 MHz Keramik
  // 10 MHz : 256 --> 39.0625 kHz fr PLL (oder eben FLL)
  GPIOE_PCOR = 1<<24;                 // "monitor"
  GPIOE_PCOR = 1<<25;
  OSC_CR = 3;                         // 8+16pF ein, Rest aus
  MCG_C2 = (1<<4) | (1<<3) | (1<<2);  // Range=1, HGain, OSCrequest
  MCG_C1 = (2<<6) | (3<<3);           // CLKS=2, FRDIV=3 (OSC:256-->39.0625 kHz)

  // warten, bis OSCINIT0 nach 4096 Takten auf 1 geht
  while (!(MCG_S & 2))
  { GPIOE_PCOR = 1<<24;               // damit man was sieht
    GPIOE_PSOR = 1<<24;
  };

  // warten, bis IREFST auf 0 geht. Also Takt fr PLL vom Quarz kommt
  while (MCG_S & 16)
  { GPIOE_PSOR = 1<<25;               // damit man was sieht
  };

  // warten, bis CLKST auf 2 geht. Also MCGOUTCLK erstmal direkt vom Quarz getrieben wird
  // (auf den Ausgang der PLL schalten wir erst spter)
  while ((MCG_S & 12) != 8)
  { GPIOE_PCOR = 1<<25;               // damit man was sieht
    GPIOE_PSOR = 1<<25;
  };

  // So, der System-Takt kommt erstmal direkt vom Quarz (10 MHz) und die PLL sieht 39 kHz.
  // die CPU vertrgt 100 MHz, der BUS vertrgt 50 MHz und der Flash vertrgt 25 MHz
  SIM_CLKDIV1 = (0<<28) | (1<<24) | (7<<16);  // :1-->CPU, :2-->BUS, :4-->Flash
  SIM_FCFG1   = 0;                            // Flash immer enabled

  // nun Multiplikator fr PLL auf 2560 setzen: (10 MHz / 256) * 2560 --> 100 MHz
  MCG_C4      = (3<<5);                       // Faktor ist 2560

  // bei MK02FN scheint es kein Statusbit zu geben, was anzeigt, ob die FLL eingerastet ist.
  // also warten wir pauschal eine Weile
  i = 10000;
  while(--i)
  { GPIOE_PCOR = 1<<24;               // damit man was sieht
    GPIOE_PSOR = 1<<24;
  };

  // und mutig in die 100 MHz springen!
  MCG_C1 = (0<<6) | (3<<3);           // CLKS=0, FRDIV=3 (OSC:256-->39.0625 kHz und das mal 2560 -> 100 MHz)

}

 /* ende */
